@@ -1,5 +1,6 @@
 use clap::{Parser, Subcommand};
 use clap_complete::Shell;
+use core::fmt::{Display, Formatter};
 
 pub const fn get_styles() -> clap::builder::Styles {
     clap::builder::Styles::styled()
@@ -43,7 +44,7 @@ pub trait Process {
     async fn process(self) -> anyhow::Result<i32>;
 }
 
-#[derive(Parser, Debug)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Parser)]
 #[clap(version, styles=get_styles())]
 pub struct Args {
     #[arg(long = "generate", value_enum)]
@@ -62,13 +63,13 @@ pub struct Args {
 const PYTHON_HELP_TEXT: &str =
     "Python version or executable to use, e.g. `3.12`, `python3.12`, `/usr/bin/python3.12`";
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct SetupOptions {
-    #[clap(long, help = "Don't update $PATH in .bashrc")]
+    #[clap(long, help = "Don't update $PATH in .bashrc/.zshrc")]
     pub skip_ensurepath: bool,
-    #[clap(long, help = "Don't enable completions via .bashrc")]
+    #[clap(long, help = "Don't enable completions via .bashrc/.zshrc")]
     pub skip_completions: bool,
-    #[clap(long, help = "Don't enable `uvenv activate` via .bashrc")]
+    #[clap(long, help = "Don't enable `uvenv activate` via .bashrc/.zshrc")]
     pub skip_activate: bool,
     #[clap(
         short,
@@ -78,7 +79,7 @@ pub struct SetupOptions {
     pub force: bool,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct ListOptions {
     #[clap(short, long, help = "Short output", conflicts_with_all = ["verbose"])]
     pub short: bool,
@@ -101,7 +102,7 @@ pub struct ListOptions {
     pub venv_names: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct InstallOptions {
     pub package_name: String,
     #[clap(
@@ -120,7 +121,7 @@ pub struct InstallOptions {
     pub with: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct CreateOptions {
     pub venv_name: String,
     #[clap(long, help = PYTHON_HELP_TEXT)]
@@ -131,14 +132,14 @@ pub struct CreateOptions {
     pub force: bool,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct ActivateOptions {
     pub venv_name: String,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct UpgradeOptions {
-    pub package_name: String,
+    pub package_names: Vec<String>,
     #[clap(short = 'f', long, help = "Ignore previous version constraint")]
     pub force: bool,
     #[clap(long, help = "Don't also upgrade injected packages")]
@@ -147,7 +148,7 @@ pub struct UpgradeOptions {
     pub no_cache: bool,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct UninstallOptions {
     pub package_name: String,
     #[clap(
@@ -158,7 +159,7 @@ pub struct UninstallOptions {
     pub force: bool,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct UninstallAllOptions {
     #[clap(
         short = 'f',
@@ -170,7 +171,7 @@ pub struct UninstallAllOptions {
     pub venv_names: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct ReinstallOptions {
     pub package: String,
     #[clap(long, help = PYTHON_HELP_TEXT)]
@@ -188,7 +189,7 @@ pub struct ReinstallOptions {
     pub editable: bool,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct ReinstallAllOptions {
     #[clap(long, help = PYTHON_HELP_TEXT)]
     pub python: Option<String>,
@@ -207,7 +208,7 @@ pub struct ReinstallAllOptions {
     pub venv_names: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct UpgradeAllOptions {
     #[clap(short = 'f', long, help = "Ignore previous version constraint")]
     pub force: bool,
@@ -219,7 +220,7 @@ pub struct UpgradeAllOptions {
     pub venv_names: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct RunOptions {
     pub package_name: String,
     #[clap(long, help = "Run without `uv` cache")]
@@ -238,31 +239,200 @@ pub struct RunOptions {
     pub args: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct RunuvOptions {
     pub venv: String,
     pub uv_args: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct RunpipOptions {
     pub venv: String,
     pub pip_args: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct RunpythonOptions {
     pub venv: String,
     pub python_args: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, clap::ValueEnum)]
+pub enum OutputFormat {
+    #[default]
+    #[expect(clippy::upper_case_acronyms, reason = "Json just looks weird")]
+    JSON,
+    #[expect(clippy::upper_case_acronyms, reason = "Toml just looks weird")]
+    TOML,
+    Binary,
+}
+
+// impl OutputFormat {
+//     pub fn to_string(&self) -> String {
+//         match self {
+//             Self::JSON => "json".to_owned(),
+//             Self::TOML => "toml".to_owned(),
+//             Self::Binary => "binary".to_owned(),
+//         }
+//     }
+//     pub fn from_str(string: &str) -> Self {
+//         match string.to_lowercase().as_ref() {
+//             "toml" => Self::TOML,
+//             "json" => Self::JSON,
+//             "binary" | "msgpack" => Self::Binary,
+//             other => {
+//                 eprintln!("Unexpected format {other}! Using `toml` instead.");
+//                 Self::TOML
+//             },
+//         }
+//     }
+// }
+
+// includes to_string:
+impl Display for OutputFormat {
+    #[expect(
+        clippy::min_ident_chars,
+        reason = "The argument is called `f` in the trait."
+    )]
+    fn fmt(
+        &self,
+        f: &mut Formatter<'_>,
+    ) -> core::fmt::Result {
+        f.write_str(match self {
+            Self::JSON => "json",
+            Self::TOML => "toml",
+            Self::Binary => "binary",
+        })
+    }
+}
+
+impl From<String> for OutputFormat {
+    fn from(string: String) -> Self {
+        // Self::from_str(&value)
+        match string.to_lowercase().as_ref() {
+            "toml" => Self::TOML,
+            "json" => Self::JSON,
+            "binary" | "msgpack" => Self::Binary,
+            other => {
+                eprintln!("Unexpected format {other}! Using `toml` instead.");
+                Self::TOML
+            },
+        }
+    }
+}
+impl From<OutputFormat> for String {
+    fn from(value: OutputFormat) -> Self {
+        value.to_string()
+    }
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
+pub struct FreezeOptions {
+    /// The filename of the lockfile to generate. Defaults to `uvenv.lock`.
+    #[clap(
+        long,
+        default_value = "uvenv.lock",
+        help = "The filename of the lockfile to generate"
+    )]
+    pub filename: String,
+    /// The version of the dependencies to freeze. Defaults to `latest`.
+    #[clap(
+        long,
+        default_value = "latest",
+        help = "The version of the lockfile format [options: latest (1)]",
+        value_parser = validate_version
+    )]
+    pub version: Option<String>,
+    /// A list of dependencies to include in the lockfile. Conflicts with `exclude`.
+    #[clap(
+        long,
+        conflicts_with = "exclude",
+        help = "A list of dependencies to include in the lockfile"
+    )]
+    pub include: Vec<String>,
+    /// A list of dependencies to exclude from the lockfile. Conflicts with `include`.
+    #[clap(
+        long,
+        conflicts_with = "include",
+        help = "A list of dependencies to exclude from the lockfile"
+    )]
+    pub exclude: Vec<String>,
+    /// The output format of the frozen dependencies. Defaults to `toml`.
+    #[clap(
+        long,
+        default_value = "toml",
+        help = "The output format of the frozen dependencies"
+    )]
+    pub format: OutputFormat,
+}
+
+fn validate_version(value: &str) -> Result<String, String> {
+    match value {
+        "1" | "latest" => Ok("1".to_owned()), // also convert 'latest' into '1'
+
+        #[cfg(debug_assertions)]
+        "0" => Ok("0".to_owned()),
+
+        _ => Err("Invalid version. Only '1' and 'latest' are supported.".to_owned()),
+    }
+}
+
+/// Options for the thaw command.
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
+pub struct ThawOptions {
+    /// The filename of the lockfile to use..
+    #[clap(
+        long,
+        default_value = "",
+        help = "The filename of the lockfile to use (default: search in uvenv.lock, uvenv.toml, uvenv.json)"
+    )]
+    pub filename: String,
+    /// A list of dependencies to include when thawing. Conflicts with `exclude`.
+    #[clap(
+        long,
+        conflicts_with = "exclude",
+        help = "A list of dependencies to include when thawing"
+    )]
+    pub include: Vec<String>,
+    /// A list of dependencies to exclude when thawing. Conflicts with `include`.
+    #[clap(
+        long,
+        conflicts_with = "include",
+        help = "A list of dependencies to exclude when thawing"
+    )]
+    pub exclude: Vec<String>,
+    /// Whether to remove the whole current environment before thawing. Defaults to `false`.
+    #[clap(
+        long,
+        default_value = "false",
+        help = "Remove the current environment before thawing",
+        conflicts_with = "skip_current"
+    )]
+    pub remove_current: bool,
+    /// Whether to overwrite existing dependencies when thawing. Defaults to `true`.
+    #[clap(
+        long,
+        default_value = "false",
+        help = "Don't overwrite existing dependencies when thawing",
+        conflicts_with = "remove_current"
+    )]
+    pub skip_current: bool,
+
+    #[clap(
+        long,
+        default_value = "frozen",
+        help = "Which version of Python to use when thawing. [Options: frozen (default; use versions defined in lockfile), ignore (use default Python), <version> (specific Python version, e.g. 3.12, python3.12)]"
+    )]
+    pub python: String,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct EnsurepathOptions {
     #[clap(long, short, help = "Force update")]
     pub force: bool,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct InjectOptions {
     pub into: String,
     pub package_specs: Vec<String>,
@@ -270,18 +440,20 @@ pub struct InjectOptions {
     #[clap(long, help = "Run without `uv` cache")]
     pub no_cache: bool,
 }
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct UnInjectOptions {
     pub outof: String,
     pub package_specs: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct CheckOptions {
     #[clap(long, help = "Don't check if scripts are installed correctly.")]
     pub skip_scripts: bool,
     #[clap(long, help = "Don't check for updates", conflicts_with_all = ["show_prereleases", "ignore_constraints"])]
     pub skip_updates: bool,
+    #[clap(long, help = "Don't check if scripts still have valid interpreter.")]
+    pub skip_broken: bool,
     #[clap(long, help = "Show prerelease updates", conflicts_with_all = ["skip_updates"])]
     pub show_prereleases: bool,
     #[clap(long, help="Ignore version constraints when checking updates", conflicts_with_all = ["skip_updates"])]
@@ -293,32 +465,32 @@ pub struct CheckOptions {
     pub venv_names: Vec<String>,
 }
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct CompletionsOptions {
-    #[clap(long, short, help = "Add to ~/.bashrc")]
+    #[clap(long, short, help = "Add to ~/.bashrc (or ~/.zshrc)")]
     pub install: bool,
     // todo: support others than bash
 }
 
-#[derive(Subcommand, Debug)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Parser)]
 pub enum Commands {
     #[clap(about = "Setup additional (bash-specific) functionality.")]
     Setup(SetupOptions),
-    #[clap(about = "List packages and apps installed with uvenv.")]
+    #[clap(aliases=["ls"], about = "List packages and apps installed with uvenv.")]
     List(ListOptions),
-    #[clap(about = "Install a package (by pip name).")]
+    #[clap(aliases=["i"], about = "Install a package (by pip name).")]
     Install(InstallOptions),
-    #[clap(about = "Create a new (empty) virtualenv")]
+    #[clap(aliases=["c"], about = "Create a new (empty) virtualenv")]
     Create(CreateOptions),
     #[clap(about = "Activate a uvenv-managed virtualenv (bash only)")]
     Activate(ActivateOptions),
-    #[clap(about = "Upgrade a package.")]
+    #[clap(aliases=["u", "update"], about = "Upgrade a package.")]
     Upgrade(UpgradeOptions),
     #[clap(about = "Upgrade all uvenv-installed packages.")]
     UpgradeAll(UpgradeAllOptions),
-    #[clap(aliases = &["delete", "remove"], about = "Uninstall a package (by pip name).")]
+    #[clap(aliases = &["delete", "remove", "rm"], about = "Uninstall a package (by pip name).")]
     Uninstall(UninstallOptions),
-    #[clap(about = "Uninstall all uvenv-installed packages.")]
+    #[clap(aliases = &["remove-all", "delete-all"], about = "Uninstall all uvenv-installed packages.")]
     UninstallAll(UninstallAllOptions),
     #[clap(
         about = "Uninstall a package (by pip name) and re-install from the original spec (unless a new spec is supplied)."
@@ -326,14 +498,14 @@ pub enum Commands {
     Reinstall(ReinstallOptions),
     #[clap(about = "Reinstall all uvenv-installed packages.")]
     ReinstallAll(ReinstallAllOptions),
-    #[clap(about = "Install additional packages to a virtual environment managed by uvenv.")]
+    #[clap(aliases=["ij"], about = "Install additional packages to a virtual environment managed by uvenv.")]
     Inject(InjectOptions),
-    #[clap(aliases = &["eject"], about="Uninstall additional packages from a virtual environment managed by uvenv. (alias: `eject`)")]
+    #[clap(aliases = &["eject", "ej"], about="Uninstall additional packages from a virtual environment managed by uvenv. (alias: `eject`)")]
     Uninject(UnInjectOptions),
     #[clap(about = "Check for possible issues and updates.")]
     Check(CheckOptions),
 
-    #[clap(about = "Run a package in a temporary virtual environment.")]
+    #[clap(aliases=["x"], about = "Run a package in a temporary virtual environment.")]
     Run(RunOptions),
     #[clap(about = "Run 'uv' in the right venv.")]
     Runuv(RunuvOptions),
@@ -341,22 +513,23 @@ pub enum Commands {
     Runpip(RunpipOptions),
     #[clap(about = "Run 'python' in the right venv.")]
     Runpython(RunpythonOptions),
+
+    #[clap(about = "Create a lock file of all installed apps, which can be reinstalled via `thaw`")]
+    Freeze(FreezeOptions),
+
+    #[clap(about = "Install applications from a lockfile (usually `uvenv.lock`)")]
+    Thaw(ThawOptions),
+
     #[clap(
-        about = "Update ~/.bashrc with a PATH that includes the local bin directory that uvenv uses."
+        about = "Update ~/.bashrc (or ~/.zshrc) with a PATH that includes the local bin directory that uvenv uses."
     )]
     Ensurepath(EnsurepathOptions),
     #[clap(about = "Use --install to install the autocomplete script (bash).")]
     Completions(CompletionsOptions),
-
     #[clap(subcommand, about = "Manage uvenv itself.")]
     Self_(SelfCommands),
 }
 
-// todo `uvenv check`:
-//   - show missing metadata
-//   - show packages with updates
-//   - show packages with script issues
-
 impl Process for Commands {
     async fn process(self) -> anyhow::Result<i32> {
         match self {
@@ -378,14 +551,23 @@ impl Process for Commands {
             Self::Completions(opts) => opts.process().await,
             Self::Run(opts) => opts.process().await,
             Self::Setup(opts) => opts.process().await,
+            Self::Freeze(opts) => opts.process().await,
+            Self::Thaw(opts) => opts.process().await,
             Self::Create(opts) => opts.process().await,
             Self::Self_(opts) => opts.process().await,
             Self::Check(opts) => opts.process().await,
         }
     }
 }
+#[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
+pub struct SelfLinkOptions {
+    #[clap(long, help = "Overwrite current symlink?")]
+    pub force: bool,
+    #[clap(long, help = "Don't produce output")]
+    pub quiet: bool,
+}
 
-#[derive(Debug, Parser)]
+#[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
 pub struct SelfUpdateOptions {
     #[clap(long, help = "Update without also updating uv")]
     pub without_uv: bool,
@@ -393,27 +575,48 @@ pub struct SelfUpdateOptions {
     pub without_patchelf: bool,
 }
 
-#[derive(Debug, Parser)]
-pub struct SelfChangelogOptions {}
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
+pub struct SelfChangelogOptions;
 
-#[derive(Debug, Parser)]
-pub struct SelfMigrateOptions {}
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
+pub struct SelfMigrateOptions;
 
-#[derive(Debug, Parser)]
-pub struct SelfVersionOptions {}
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
+pub struct SelfInfoOptions {
+    #[clap(long, default_value = "true", help = "Show info about uvenv in a colored format (default)", conflicts_with_all = ["simple", "json", "toml"])]
+    pub fancy: bool,
 
-#[derive(Subcommand, Debug)]
+    #[clap(short, long, help = "Show info about uvenv in a basic format", conflicts_with_all = ["fancy", "json", "toml"])]
+    pub simple: bool,
+
+    #[clap(short, long, help = "Show info about uvenv in JSON format", conflicts_with_all = ["fancy", "simple", "toml"])]
+    pub json: bool,
+
+    #[clap(short, long, help = "Show info about uvenv in TOML format", conflicts_with_all = ["fancy", "simple", "json"])]
+    pub toml: bool,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Default, Parser)]
+pub struct SelfVersionOptions;
+
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Subcommand)]
 pub enum SelfCommands {
     #[clap(about = "Update the current installation of uvenv (and optionally uv).")]
     Update(SelfUpdateOptions),
 
+    #[clap(about = "Create a symlink for the current location of `uvenv` to ~/.local/bin/uvenv.")]
+    Link(SelfLinkOptions),
+
     #[clap(about = "Show the uvenv changelog")]
     Changelog(SelfChangelogOptions),
 
     #[clap(about = "Migrate installed environments and commands from `uvx` to `uvenv`")]
     Migrate(SelfMigrateOptions),
 
-    #[clap(about = "Show version info about uvenv and it's dependencies")]
+    #[clap(about = "Show info about uvenv and it's dependencies")]
+    Info(SelfInfoOptions),
+
+    #[clap(about = "(deprecated in favor of `self info`)")]
     Version(SelfVersionOptions),
 }
 
@@ -421,8 +624,10 @@ impl Process for SelfCommands {
     async fn process(self) -> anyhow::Result<i32> {
         match self {
             Self::Update(opts) => opts.process().await,
+            Self::Link(opts) => opts.process().await,
             Self::Changelog(opts) => opts.process().await,
             Self::Migrate(opts) => opts.process().await,
+            Self::Info(opts) => opts.process().await,
             Self::Version(opts) => opts.process().await,
         }
     }
