@@ -2,7 +2,7 @@ use crate::cli::{Process, UpgradeAllOptions};
 use crate::commands::list::list_packages;
 use crate::commands::upgrade::upgrade_package;
 use crate::metadata::LoadMetadataConfig;
-use anyhow::{anyhow, Context};
+use anyhow::{Context, anyhow};
 
 pub async fn upgrade_all(
     force: bool,
@@ -35,16 +35,13 @@ pub async fn upgrade_all(
 
 impl Process for UpgradeAllOptions {
     async fn process(self) -> anyhow::Result<i32> {
-        match upgrade_all(
+        upgrade_all(
             self.force,
             self.no_cache,
             self.skip_injected,
             &self.venv_names,
         )
         .await
-        {
-            Ok(()) => Ok(0),
-            Err(msg) => Err(msg),
-        }
+        .map(|()| 0)
     }
 }
