@@ -1,50 +1,49 @@
+use crate::commands::self_update::extract_version;
 use crate::pypi::get_latest_version;
 use crate::symlinks::check_symlink;
-use crate::uv::{uv_get_installed_version, uv_venv, Helpers};
-use anyhow::anyhow;
+use crate::uv::{Helpers, PythonSpecifier, uv_freeze, uv_get_installed_version, uv_venv};
+use core::cmp::Ordering;
+use core::fmt::Write;
+use core::str::FromStr;
 use itertools::Itertools;
 use owo_colors::OwoColorize;
-use pep440_rs::{Version, VersionSpecifier};
-use pep508_rs::Requirement;
 use serde::{Deserialize, Serialize};
-use std::collections::{HashMap, HashSet};
-use std::fs::remove_dir_all;
+use std::collections::{BTreeMap, HashSet};
 use std::path::{Path, PathBuf};
-use std::str::FromStr;
-use tokio::fs::{create_dir_all, File};
+use tokio::fs::{File, create_dir_all};
 use tokio::io::AsyncReadExt;
 use tokio::io::AsyncWriteExt;
+use uv_pep440::{Version, VersionSpecifier};
+use uv_pep508::Requirement;
 use uv_python::PythonEnvironment;
 
-const BIN_DIR: &str = ".local/bin";
-const WORK_DIR: &str = ".local/uvenv";
 const INDENT: &str = "    ";
 
 // tells 'file' that a .metadata file is 'data' (instead of making it guess)
 //                           U     V     X    SOH  version(2)  STX (padding):
-const MAGIC_HEADER: &[u8] = &[0x55, 0x56, 0x58, 0x01, 0x32, 0x04, 0x00]; // hex, 7 bytes
+const MAGIC_HEADER_V2: &[u8] = &[0x55, 0x56, 0x58, 0x01, 0x32, 0x04, 0x00]; // hex, 7 bytes
 
 pub fn get_home_dir() -> PathBuf {
-    if cfg!(test) {
-        let test_dir = std::env::temp_dir().join("uvenv-test");
-        // fixme:
-        let _ = dbg!(remove_dir_all(&test_dir));
-        let _ = dbg!(std::fs::create_dir_all(&test_dir));
-        test_dir
-    } else {
-        home::home_dir().expect("Failed to get home directory")
-    }
+    // if snap: $HOME is now ~/snap/uvenv/<revision>/
+    home::home_dir().expect("Failed to get home directory")
 }
 
 pub fn get_bin_dir() -> PathBuf {
     let home_dir = get_home_dir();
-    home_dir.join(BIN_DIR)
+
+    if cfg!(feature = "snap") {
+        // $SNAP/bin, which exists in snap's $PATH
+        home_dir.join("bin")
+    } else {
+        home_dir.join(".local/bin")
+    }
 }
 
 pub async fn ensure_bin_dir() -> PathBuf {
     let bin_dir = get_bin_dir();
 
     if !bin_dir.exists() {
+        // may fail I guess:
         let _ = create_dir_all(&bin_dir).await;
     }
 
@@ -53,7 +52,15 @@ pub async fn ensure_bin_dir() -> PathBuf {
 
 pub fn get_work_dir() -> PathBuf {
     let home_dir = get_home_dir();
-    home_dir.join(WORK_DIR)
+    if cfg!(feature = "snap") {
+        // on snap, store everything in the 'common' ($SNAP_USER_COMMON)
+        // dir because that's scoped to the snap and not changed across revisions
+        std::env::var("SNAP_USER_COMMON")
+            .map(PathBuf::from)
+            .unwrap_or(home_dir)
+    } else {
+        home_dir.join(".local/uvenv")
+    }
 }
 
 pub fn get_venv_dir() -> PathBuf {
@@ -69,12 +76,91 @@ pub fn version_0() -> Version {
     Version::from_str("0.0.0").expect("Version 0.0.0 should be parseable.")
 }
 
-#[derive(Debug, PartialEq, Eq, Deserialize, Serialize)] // dbg_pls::DebugPls
+#[derive(Debug)]
+pub enum FileType {
+    Shebang { interpreter: String },
+    Binary,
+    Unusable(String),
+}
+
+async fn identify_file(path: &Path) -> anyhow::Result<FileType> {
+    if !path.exists() {
+        return Ok(FileType::Unusable("No Such File".to_owned()));
+    }
+
+    if path.is_dir() {
+        return Ok(FileType::Unusable("Directory".to_owned()));
+    }
+
+    let mut file = File::open(path).await?;
+    let mut buffer = [0_u8; 1024];
+    let n = file.read(&mut buffer).await?;
+
+    if n == 0 {
+        return Ok(FileType::Unusable("Empty".to_owned()));
+    }
+
+    let bytes = &buffer.get(..n).unwrap_or_default();
+
+    // Check for shebang (#!)
+    if bytes.starts_with(b"#!") {
+        let interpreter = String::from_utf8_lossy(bytes)
+            .lines()
+            .next()
+            .unwrap_or("")
+            .trim()
+            .to_owned();
+
+        return Ok(FileType::Shebang { interpreter });
+    }
+
+    // Check if file is binary
+    let is_binary = bytes.iter().any(|&byte| byte == 0 || byte > 127);
+    if is_binary {
+        return Ok(FileType::Binary);
+    }
+
+    Ok(FileType::Unusable("No Shebang".to_owned()))
+}
+
+pub fn check_shebang(shebang: &str) -> bool {
+    let Some(stripped_path) = shebang.strip_prefix("#!") else {
+        return false;
+    };
+    let interpreter_path = Path::new(stripped_path);
+
+    // should chmod also be checked?
+    interpreter_path.exists()
+}
+
+pub async fn check_executable(script_name: &str) -> Option<String> {
+    // todo
+    let script_path = ensure_bin_dir().await.join(script_name);
+    let Ok(result) = identify_file(&script_path).await else {
+        return Some("Could not identify script type".to_owned());
+    };
+    match result {
+        FileType::Shebang { interpreter } => {
+            if check_shebang(&interpreter) {
+                None
+            } else {
+                Some("Interpreter does not exist".to_owned())
+            }
+        },
+        FileType::Binary => {
+            // should chmod also be checked?
+            None
+        },
+        FileType::Unusable(reason) => Some(reason),
+    }
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Default, Serialize, Deserialize)]
 pub struct Metadata {
     // order is important!!
     pub name: String,
     #[serde(default)]
-    pub scripts: HashMap<String, bool>,
+    pub scripts: BTreeMap<String, bool>,
     pub install_spec: String,
     #[serde(default)]
     pub extras: HashSet<String>,
@@ -91,14 +177,35 @@ pub struct Metadata {
     pub available_version: String,
     #[serde(default)]
     pub outdated: bool,
+    #[serde(default)]
+    pub python_spec: Option<String>,
+}
+
+impl PartialOrd for Metadata {
+    fn partial_cmp(
+        &self,
+        other: &Self,
+    ) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for Metadata {
+    fn cmp(
+        &self,
+        other: &Self,
+    ) -> Ordering {
+        String::cmp(&self.name, &other.name)
+    }
 }
 
 impl Metadata {
     pub fn new(name: &str) -> Self {
         Self {
-            name: name.to_string(),
-            scripts: HashMap::new(),
-            install_spec: name.to_string(),
+            name: name.to_owned(),
+            scripts: BTreeMap::new(),
+            install_spec: name.to_owned(),
+            python_spec: None,
             extras: HashSet::new(),
             requested_version: String::new(),
             installed_version: String::new(),
@@ -111,14 +218,43 @@ impl Metadata {
         }
     }
 
-    #[allow(dead_code)]
+    // /// When using uvenv's install.sh, the installation is done via bash
+    // /// so the metadata msgpack is not stored. This function works as a fallback.
+    // fn for_self_install() -> Self {
+    //     Self {
+    //         name: "uvenv".to_owned(),
+    //         ..Default::default()
+    //     }
+    // }
+
+    /// return `PythonEnvironment` (if available) for this metadata
+    pub fn venv(&self) -> Option<PythonEnvironment> {
+        let venv_dir = venv_path(&self.name);
+
+        PythonSpecifier::PathBuf(&venv_dir).into_environment().ok()
+    }
+
+    #[expect(dead_code, reason = "Useful variant of `installed_version_parsed`")]
     pub fn requested_version_parsed(&self) -> Version {
         Version::from_str(&self.requested_version).unwrap_or_else(|_| version_0())
     }
+
     pub fn installed_version_parsed(&self) -> Version {
         Version::from_str(&self.installed_version).unwrap_or_else(|_| version_0())
     }
 
+    // check actual currently installed version
+    pub fn find_currently_installed_version_raw(&self) -> Option<String> {
+        let venv = self.venv()?;
+        let frozen = uv_freeze(&venv).ok()?;
+
+        extract_version(&frozen, &self.name)
+    }
+    pub fn find_currently_installed_version(&self) -> Option<Version> {
+        let version_raw = self.find_currently_installed_version_raw()?;
+        Version::from_str(&version_raw).ok()
+    }
+
     pub fn find(req: &Requirement) -> Self {
         let mut empty = Self::new(req.name.as_ref());
 
@@ -159,7 +295,7 @@ impl Metadata {
         }
 
         let venv = match maybe_venv {
-            Some(v) => v,
+            Some(venv) => venv,
             None => match uv_venv(None) {
                 Ok(venv) => {
                     // black magic fuckery to
@@ -181,7 +317,7 @@ impl Metadata {
     pub async fn for_owned_dir(
         dirname: PathBuf,
         config: &LoadMetadataConfig,
-    ) -> anyhow::Result<Self> {
+    ) -> Option<Self> {
         let meta_path = dirname.join(".metadata");
 
         Self::for_file(&meta_path, config).await.map_or_else(
@@ -191,12 +327,14 @@ impl Metadata {
                     .and_then(|fname| fname.to_str())
                     .unwrap_or_default();
 
-                Err(anyhow!(
-                    "Metadata for '{}' could not be loaded.",
-                    venv_name.red()
-                ))
+                if venv_name != "uvenv" {
+                    // exception case for uvenv itself
+                    eprintln!("Metadata for '{}' could not be loaded.", venv_name.red());
+                }
+
+                None
             },
-            Ok,
+            Some,
         )
     }
 
@@ -249,7 +387,12 @@ impl Metadata {
 
         if let Some(latest_version) = get_latest_version(&self.name, !prereleases, constraint).await
         {
-            let installed_version = self.installed_version_parsed();
+            let installed_version = self
+                // first try getting latest from venv:
+                .find_currently_installed_version()
+                // fallback to cached version:
+                .unwrap_or_else(|| self.installed_version_parsed());
+
             self.available_version = latest_version.to_string();
             self.outdated = latest_version > installed_version;
         }
@@ -269,10 +412,22 @@ impl Metadata {
             .scripts
             .iter()
             //                                if True, the script is valid -> skip from filter_map
-            .filter_map(|(k, v)| if *v { None } else { Some(k.to_owned()) })
+            .filter_map(|(key, val)| if *val { None } else { Some(key.to_owned()) })
             .collect()
     }
 
+    pub async fn broken_scripts(&self) -> Vec<String> {
+        let mut result = vec![];
+
+        for script in self.scripts.keys() {
+            if let Some(reason) = check_executable(script).await {
+                result.push(reason);
+            }
+        }
+
+        result
+    }
+
     pub fn format_installed_version(&self) -> String {
         if self.outdated {
             self.installed_version.red().to_string()
@@ -281,11 +436,16 @@ impl Metadata {
         }
     }
 
+    // #[expect(clippy::use_debug, reason = "Print the struct on --verbose")]
+    pub fn format_debug(&self) -> String {
+        format!("{:#?}", &self)
+    }
+
     pub fn format_short(&self) -> String {
         format!("- {} {}", self.name, self.format_installed_version())
     }
 
-    #[allow(dead_code)]
+    #[expect(dead_code, reason = "Useful variant of `vec_injected`")]
     pub fn vec_extras(&self) -> Vec<&str> {
         self.extras.iter().map(AsRef::as_ref).collect()
     }
@@ -297,52 +457,70 @@ impl Metadata {
     pub fn format_extras(&self) -> String {
         self.extras
             .iter()
-            .map(|k| format!("'{}'", k.green()))
+            .map(|extra| format!("'{}'", extra.green()))
             .join(",")
     }
 
     pub fn format_injected(&self) -> String {
         self.injected
             .iter()
-            .map(|k| format!("'{}'", k.green()))
+            .map(|dep| format!("'{}'", dep.green()))
             .join(", ")
     }
 
-    pub fn format_human(&self) -> String {
+    pub fn format_human(&self) -> anyhow::Result<String> {
         let mut result = format!("- {}", self.name);
 
         if !self.extras.is_empty() {
-            result.push_str(&format!("[{}]", self.format_extras()));
+            // result.push_str(&format!("[{}]", self.format_extras()));
+            write!(result, "[{}]", self.format_extras())?;
         }
 
         if !self.requested_version.is_empty() {
-            result.push_str(&format!(" {}", self.requested_version.cyan()));
+            // result.push_str(&format!(" {}", self.requested_version.cyan()));
+            write!(result, " {}", self.requested_version.cyan())?;
         }
 
         if self.editable {
-            result.push_str(&format!(" {}", "--editable".yellow()));
+            // result.push_str(&format!(" {}", "--editable".yellow()));
+            write!(result, " {}", "--editable".yellow())?;
         }
 
         result.push('\n');
 
-        result.push_str(&format!(
-            "{}Installed Version: {} on {}.\n",
+        // result.push_str(&format!(
+        //     "{}Installed Version: {} on {}.\n",
+        //     INDENT,
+        //     self.format_installed_version(),
+        //     self.python.bright_blue()
+        // ));
+        writeln!(
+            result,
+            "{}Installed Version: {} on {}.",
             INDENT,
             self.format_installed_version(),
             self.python.bright_blue()
-        ));
+        )?;
 
         if self.outdated && !self.available_version.is_empty() {
-            result.push_str(&format!(
-                "{}Available Version: {}.\n",
+            // result.push_str(&format!(
+            //     "{}Available Version: {}.\n",
+            //     INDENT,
+            //     self.available_version.green(),
+            // ));
+
+            writeln!(
+                result,
+                "{}Available Version: {}.",
                 INDENT,
                 self.available_version.green(),
-            ));
+            )?;
         }
 
         if !self.injected.is_empty() {
             let formatted_injects = self.format_injected();
-            result.push_str(&format!("{INDENT}Injected Packages: {formatted_injects}\n"));
+            // result.push_str(&format!("{INDENT}Injected Packages: {formatted_injects}\n"));
+            writeln!(result, "{INDENT}Injected Packages: {formatted_injects}")?;
         }
 
         let formatted_scripts = self
@@ -356,9 +534,10 @@ impl Metadata {
                 }
             })
             .join(" | ");
-        result.push_str(&format!("{INDENT}Scripts: {formatted_scripts}"));
+        // result.push_str(&format!("{INDENT}Scripts: {formatted_scripts}"));
+        writeln!(result, "{INDENT}Scripts: {formatted_scripts}")?;
 
-        result
+        Ok(result)
     }
 }
 
@@ -367,15 +546,15 @@ pub fn strip_header(buf: &mut Vec<u8>) {
     // postponed: the current header is 7 chars long.
     //            the version should be ignored for starts_with,
     //            and if the length should change, this must be 'match'ed based on the version
-    if buf.starts_with(MAGIC_HEADER) {
-        let _ = buf.drain(0..MAGIC_HEADER.len());
+    if buf.starts_with(MAGIC_HEADER_V2) {
+        let _ = buf.drain(0..MAGIC_HEADER_V2.len());
     }
 }
 
 /// Prepend the `MAGIC_HEADER` to a buffer
-fn add_header(buf: &mut Vec<u8>) {
-    let mut new_buf = Vec::with_capacity(MAGIC_HEADER.len() + buf.len());
-    new_buf.extend_from_slice(MAGIC_HEADER);
+pub fn add_header(buf: &mut Vec<u8>) {
+    let mut new_buf = Vec::with_capacity(MAGIC_HEADER_V2.len() + buf.len());
+    new_buf.extend_from_slice(MAGIC_HEADER_V2);
     new_buf.append(buf);
     *buf = new_buf;
 }
@@ -392,9 +571,9 @@ fn add_header(buf: &mut Vec<u8>) {
 ///
 ///       Ok(metadata)
 ///     }
-pub async fn load_generic_msgpack<'a, T: serde::Deserialize<'a>>(
+pub async fn load_generic_msgpack<'buf, T: serde::Deserialize<'buf>>(
     filename: &Path,
-    buf: &'a mut Vec<u8>,
+    buf: &'buf mut Vec<u8>,
 ) -> anyhow::Result<T> {
     // Open the msgpack file
     let mut file = File::open(filename).await?;
@@ -404,12 +583,13 @@ pub async fn load_generic_msgpack<'a, T: serde::Deserialize<'a>>(
     strip_header(buf);
 
     // Read the contents of the file into a Metadata struct
-    let metadata: T = rmp_serde::decode::from_slice(&buf[..])?;
+    let metadata: T = rmp_serde::decode::from_slice(buf)?;
 
     Ok(metadata)
 }
 
-#[allow(clippy::struct_excessive_bools)]
+#[expect(clippy::struct_excessive_bools, reason = "...")]
+#[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, Hash, Serialize, Deserialize)]
 pub struct LoadMetadataConfig {
     pub recheck_scripts: bool,
     pub updates_check: bool,
@@ -418,7 +598,7 @@ pub struct LoadMetadataConfig {
 }
 
 impl LoadMetadataConfig {
-    #[allow(dead_code)]
+    #[expect(dead_code, reason = "Useful variant in addition to `none()`")]
     pub const fn all() -> Self {
         Self {
             recheck_scripts: true,
@@ -477,21 +657,54 @@ pub async fn load_metadata(
     Ok(metadata)
 }
 
-pub async fn store_generic_msgpack<T: serde::Serialize>(
-    filename: &Path,
-    metadata: &T,
-) -> anyhow::Result<()> {
-    // Open the msgpack file
-    let mut file = File::create(filename).await?;
-
+pub async fn serialize_msgpack<T: serde::Serialize>(metadata: &T) -> anyhow::Result<Vec<u8>> {
     // Read the contents of the file into a Metadata struct
     let mut bytes = rmp_serde::encode::to_vec(metadata)?;
 
     add_header(&mut bytes);
 
-    file.write_all(&bytes).await?;
+    Ok(bytes)
+}
 
-    Ok(())
+/// Atomically writes the given bytes to the specified file.
+///
+/// This function first writes to `<filename>.tmp`, then renames it to `filename`.
+/// This approach prevents data loss from incomplete writes, as the original file is
+/// not modified until the entire write succeeds.
+///
+/// If any step fails, the temporary file is removed to avoid clutter.
+pub async fn atomic_write<P: AsRef<Path>>(
+    filename: P,
+    bytes: &[u8],
+) -> anyhow::Result<()> {
+    let tmp_filename = filename.as_ref().with_extension("tmp");
+
+    // Attempt to write and rename, cleaning up the temp file on error
+    let result = async {
+        let mut file = File::create(&tmp_filename).await?;
+        file.write_all(bytes).await?;
+        file.flush().await?;
+
+        tokio::fs::rename(&tmp_filename, filename).await?;
+
+        Ok(())
+    }
+    .await;
+
+    // Clean up tmp file if something went wrong
+    if result.is_err() {
+        let _ = tokio::fs::remove_file(&tmp_filename).await;
+    }
+
+    result
+}
+
+pub async fn store_generic_msgpack<T: serde::Serialize>(
+    filename: &Path,
+    metadata: &T,
+) -> anyhow::Result<()> {
+    let bytes = serialize_msgpack(metadata).await?;
+    atomic_write(filename, &bytes).await
 }
 
 pub async fn store_metadata(
