@@ -1,7 +1,8 @@
 use crate::helpers::fmt_error;
+use core::future::Future;
 use futures::future;
-use std::future::Future;
 
+#[expect(dead_code, reason = "May be useful again in the future")]
 pub async fn handle_promises<T, F: Future<Output = anyhow::Result<T>>>(promises: Vec<F>) -> Vec<T> {
     future::join_all(promises)
         .await
