@@ -1,16 +1,17 @@
 use crate::pip::parse_requirement;
 use crate::uv::uv_cache;
 use rkyv::{Archive, Archived, Deserialize, deserialize};
+use uv_normalize::PackageName;
 use uv_pep440::{Version, VersionSpecifier};
-use uv_pep508::{PackageName, Requirement};
+use uv_pep508::Requirement;
 use uv_pypi_types::Yanked;
 
 use rkyv::api::high::HighDeserializer;
 use std::collections::HashSet;
 use tokio::sync::Semaphore;
 use uv_client::{
-    MetadataFormat, OwnedArchive, RegistryClient, RegistryClientBuilder, SimpleMetadata,
-    SimpleMetadatum, VersionFiles,
+    BaseClientBuilder, MetadataFormat, OwnedArchive, RegistryClient, RegistryClientBuilder,
+    SimpleMetadata, SimpleMetadatum, VersionFiles,
 };
 use uv_distribution_types::IndexCapabilities;
 
@@ -52,7 +53,8 @@ impl Default for SimplePypi {
     /// Create a (default) Registry
     fn default() -> Self {
         let cache = uv_cache();
-        let inner = RegistryClientBuilder::new(cache).build();
+        let base_client = BaseClientBuilder::default();
+        let inner = RegistryClientBuilder::new(base_client, cache).build();
 
         Self(inner)
     }
@@ -167,10 +169,10 @@ pub async fn get_pypi_data_for_packagename(package_name: &PackageName) -> Option
 
     let data = client.lookup(package_name).await.ok()?;
 
-    if let Some(metadata) = data.iter().next_back() {
-        if let Some(latest) = metadata.iter().next_back() {
-            return deserialize_metadata(latest);
-        }
+    if let Some(metadata) = data.iter().next_back()
+        && let Some(latest) = metadata.iter().next_back()
+    {
+        return deserialize_metadata(latest);
     }
 
     None
