@@ -10,10 +10,11 @@ use std::path::Path;
 use std::{collections::HashSet, path::PathBuf};
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity};
-use uv_configuration::PreviewMode;
-use uv_distribution_types::{InstalledDist, Name};
+use uv_distribution_types::{InstalledDistKind, Name};
 use uv_installer::SitePackages;
-use uv_pep508::{PackageName, Requirement};
+use uv_normalize::PackageName;
+use uv_pep508::Requirement;
+use uv_preview::Preview;
 use uv_python::{
     EnvironmentPreference, Interpreter, PythonDownloads, PythonEnvironment, PythonInstallation,
     PythonPreference, PythonRequest,
@@ -21,7 +22,7 @@ use uv_python::{
 
 use uv_pep508::VersionOrUrl::VersionSpecifier;
 
-use crate::helpers::PathToString;
+use crate::helpers::{PathToString, set_env_var};
 use crate::metadata::get_work_dir;
 
 pub async fn maybe_get_uv_binary() -> Option<String> {
@@ -35,12 +36,27 @@ pub async fn get_uv_binary() -> String {
     )
 }
 
+fn apply_uv_env_settings() {
+    if cfg!(feature = "snap") {
+        let work_dir = get_work_dir();
+        let python_dir = work_dir.join("python");
+
+        // by default, uv in snap would install at
+        // ~/snap/uvenv/<revision>/.local/share/uv/python/
+        // meaning it would be moved after each update;
+        // leading to longer update times and breaking symlinks.
+        // so, we set the install dir to a fixed location (~/snap/uvenv/common/python)
+        set_env_var("UV_PYTHON_INSTALL_DIR", python_dir.to_string());
+    }
+}
+
 /// Start `uv` in a subprocess and handle its output
 /// Note: while `uv::main` exists, it's not recommended to use as an entrypoint.
 /// It also calls `exit`, stopping `uvenv` instead of returning an exit code.
 pub async fn uv<S: AsRef<OsStr>>(args: &[S]) -> anyhow::Result<bool> {
     // venv could be unavailable, use 'uv' from this library's requirement
     let script = get_uv_binary().await;
+    apply_uv_env_settings();
 
     let subcommand = args
         .first()
@@ -72,6 +88,10 @@ pub fn uv_cache() -> Cache {
     )
 }
 
+fn uv_featureflags() -> Preview {
+    Preview::default()
+}
+
 /// try to find a `PythonEnvironment` based on Cache or currently active virtualenv (`VIRTUAL_ENV`).
 pub fn uv_venv(maybe_cache: Option<Cache>) -> anyhow::Result<PythonEnvironment> {
     let cache = maybe_cache.unwrap_or_else(uv_cache);
@@ -80,8 +100,9 @@ pub fn uv_venv(maybe_cache: Option<Cache>) -> anyhow::Result<PythonEnvironment>
     let environ = PythonEnvironment::find(
         &PythonRequest::Any,                // just find me a python
         EnvironmentPreference::OnlyVirtual, // venv is always virtual
+        PythonPreference::Managed,
         &cache,
-        PreviewMode::Disabled,
+        uv_featureflags(),
     )?;
 
     Ok(environ)
@@ -94,8 +115,9 @@ pub fn environment_from_path_str(path: &str) -> anyhow::Result<PythonEnvironment
     Ok(PythonEnvironment::find(
         &PythonRequest::parse(path),
         EnvironmentPreference::ExplicitSystem, // based on above python wishes
+        PythonPreference::Managed,
         &cache,
-        PreviewMode::Disabled,
+        uv_featureflags(),
     )?)
 }
 
@@ -111,8 +133,9 @@ pub fn system_environment() -> anyhow::Result<PythonEnvironment> {
     Ok(PythonEnvironment::find(
         &PythonRequest::Any, // just find me a python
         EnvironmentPreference::OnlySystem,
+        PythonPreference::OnlySystem,
         &cache,
-        PreviewMode::Disabled,
+        uv_featureflags(),
     )?)
 }
 
@@ -143,7 +166,7 @@ pub async fn uv_search_python(python: Option<&str>) -> Option<String> {
         None,
         None,
         None,
-        PreviewMode::Disabled,
+        uv_featureflags(),
     )
     .await
     .ok()?;
@@ -196,12 +219,12 @@ pub fn uv_freeze(python: &PythonEnvironment) -> anyhow::Result<String> {
         // .filter() ?
         .sorted_unstable_by(|one, two| one.name().cmp(two.name()).then(one.version().cmp(two.version())))
     {
-        match installed_dist {
-            InstalledDist::Registry(dist) => {
+        match &installed_dist.kind {
+            InstalledDistKind::Registry(dist) => {
                 // result.push_str(&format!("{}=={}\n", dist.name(), dist.version));
                 writeln!(result, "{}=={}", dist.name(), dist.version)?;
             },
-            InstalledDist::Url(dist) => {
+            InstalledDistKind::Url(dist) => {
                 if dist.editable {
                     // result.push_str(&format!("-e {}\n", dist.url));
                     writeln!(result, "-e {}", dist.url)?;
@@ -210,15 +233,15 @@ pub fn uv_freeze(python: &PythonEnvironment) -> anyhow::Result<String> {
                     writeln!(result, "{} @ {}", dist.name(), dist.url)?;
                 }
             },
-            InstalledDist::EggInfoFile(dist) => {
+            InstalledDistKind::EggInfoFile(dist) => {
                 // result.push_str(&format!("{}=={}\n", dist.name(), dist.version));
                 writeln!(result, "{}=={}", dist.name(), dist.version)?;
             },
-            InstalledDist::EggInfoDirectory(dist) => {
+            InstalledDistKind::EggInfoDirectory(dist) => {
                 // result.push_str(&format!("{}=={}\n", dist.name(), dist.version));
                 writeln!(result, "{}=={}", dist.name(), dist.version)?;
             },
-            InstalledDist::LegacyEditable(dist) => {
+            InstalledDistKind::LegacyEditable(dist) => {
                 // result.push_str(&format!("-e {}\n", dist.target.display()));
                 writeln!(result, "-e {}", dist.target.display())?;
             },
